#include <EEPROM.h>

// Mega Drive 2 Switchless MOD
// version 1.1 -- 03-August-2019
// By Mirco Gaggiottini
//
// Hardware: arduino pro mini
// Led RGB
//
// Pinout:
// Led pin (10,11,12) (R,G,B)
// reset input:   pin 6
// reset output:  pin 7
// 50/60hz:       pin 8
// Eng/Jap:       pin 9


// how it work!
// 1) When powered: 1 blink: 50hz -  2 blinks: 60hz
// 2) When reset button is pressed:
// <400ms: normal reset
// <1400ms: toggle 50/60hz (no reset, it just switch mode between 50hz and 60hz)
// >1400ms: change region every 1 sec. The led shows the current region color, when it is released arduino apply the selected region and reset the system
//
// Legend of zel.. led:
// Led red:    Japan Mode (60hz default)
// Led blue:   USA Mode (60hz default)
// Led green:  Europe Mode (50hz default)

const int LEDr = 10;      //  LED Red
const int LEDg = 11;      //  LED Green
const int LEDb = 12;      //  LED Blue
//reset
const int ResetKey = 6;    // Input Reset key
const int ResetLine = 7;     // Output to /RESET line
//language
const int ModHz = 8;     // Output the 50/60hz (gnd/5v)
const int ModLang = 9;     // Output the Jp/En language (gnd/5v) 

int ResetDelay = 1000;  // 1000ms delay for reset key
int HzDelay = 400;  // 400ms delay for change 50/60hz

const unsigned long repeatdelay = 5; // used for debouncing
 
int CurrentLang; // which languase is select (0:EN 1:JP)
int CurrentHz; // which languase is select (0:50hz 1:60hz)
int debouncecounter = 0;       // how many times we have seen new value (for debounce)
int debouncereading;
int debounce_count;
int ResetHeld;
unsigned long TimeHeld; // amount of time Resotre is held down

int buttonDuration = 0; // for keeping track of how long restore is held down
boolean buttonHeld = 0; // for keeping track when you are holding down 
boolean Released = 0; // Keeping track when the restore key is released
boolean holdingReset = 0; // Keeping track if you are holding restore
boolean resetSystem = 0; // keep track wheter to reset
bool doOff= false; //led off memory

int buttonInput; // used to return if restore is held
unsigned long time; //used to keep track of millis output
unsigned long htime; //used to keep track of millis output
unsigned long btime; //used to keep track of bounce millis output


void setup() {
  pinMode(LEDr, OUTPUT);
  pinMode(LEDg, OUTPUT);
  pinMode(LEDb, OUTPUT);
  pinMode(ResetLine, INPUT);
  pinMode(ResetKey, INPUT);
  pinMode(ModHz, OUTPUT);
  pinMode(ModLang, OUTPUT);
  digitalWrite(ResetKey, HIGH); // pull-up
  digitalWrite(ResetLine, LOW); // keep the system reset
  pinMode(ResetLine, OUTPUT); // switch reset line to OUTPUT so it can hold it low
  digitalWrite(ResetLine, LOW); // keep the system reset

  CurrentLang = EEPROM.read(1);
  if (CurrentLang>2) {CurrentLang=0;}//0=EU 1=US 2:JP
  SetRegion(CurrentLang);
  
  CurrentHz = EEPROM.read(2);
  if (CurrentHz>1) {CurrentHz=0;}
  SetHZ(CurrentHz);
  delay(50);
  
  pinMode(ResetLine, INPUT); // set the reset pin back to high impedance
  if (CurrentHz==0){//50hz - 1 blink
      FlashLED(CurrentLang);
      delay(500);
      LEDoff();
      delay(500);
      FlashLED(CurrentLang);
  } else {//60hz - 2 blinks
      FlashLED(CurrentLang);
      delay(500);
      LEDoff();
      delay(500);
      FlashLED(CurrentLang);
      delay(500);
      LEDoff();
      delay(500);
      FlashLED(CurrentLang);
  }
   
  // 3..2..1... GO!
}

void loop() {
    buttonInput = readButton(); delay(5);
  time = millis(); 
  if (buttonInput == 1) {
    if (!buttonHeld) {
      htime = time; TimeHeld = 0; buttonHeld = 1; } //reset button is pushed
    else { 
      TimeHeld = time - htime; }
  }

  if (buttonInput == 0) {// button released

    if (buttonHeld) {
          if (TimeHeld > 0 && TimeHeld < HzDelay && !resetSystem) {doReset();}// button pressed < 400ms
          if (TimeHeld > HzDelay) {
          if (CurrentHz==1) CurrentHz=0; else CurrentHz=1;
            SetHZ(CurrentHz);
            SaveSlot(CurrentLang,CurrentHz);
            FlashLED(CurrentLang);
            doOff=false;
          }

      Released = 1; buttonHeld = 0; htime = millis(); TimeHeld = 0; //reset button not being held anymore
    } 
  }

  if (TimeHeld > HzDelay && !Released && !doOff){//time is good to change Hz... turn off led
      digitalWrite(LEDr, LOW);
      digitalWrite(LEDg, LOW);
      digitalWrite(LEDb, LOW);
      doOff=true;
  }
  
  if (TimeHeld > ResetDelay && !Released) {
    htime = millis();
    holdingReset = 1; resetSystem = 1;
      if (CurrentLang < 2) { 
        CurrentLang++;}
      else { CurrentLang = 0; 
      }
      if (TimeHeld > ResetDelay) { TimeHeld = 0;} 
      FlashLED(CurrentLang);
  }
  
  if (Released) {
    //if language is changed, reset the system
    if (resetSystem) {
      htime = millis();
      resetSystem = 0;
      holdingReset = 0;
      Released = 0;
      TimeHeld = 0;
      doOff=false;
      digitalWrite(ResetLine, LOW); 
      pinMode(ResetLine, OUTPUT);
      digitalWrite(ResetLine, LOW); 

      SetRegion(CurrentLang);
      SetHZ(CurrentHz);
      SaveSlot(CurrentLang,CurrentHz);
  
      delay(150);
      pinMode(ResetLine, INPUT); // set the reset pin back to high impedance so it can boots

    } else {
      htime = millis(); Released = 0; resetSystem = 0; holdingReset = 0; doOff=false; TimeHeld = 0;
    }
  }

}//loop

void doReset() {
      digitalWrite(ResetLine, LOW); // keep the system reset
      pinMode(ResetLine, OUTPUT);
      digitalWrite(ResetLine, LOW);
      delay(100);
      pinMode(ResetLine, INPUT); // set the reset pin back to high impedance so it can boots
      delay(50);
}

int readButton() {
 if (!digitalRead(ResetKey) && (millis() - btime >= repeatdelay)) {
  for(int i = 0; i < 10; i++)
    {
      debouncereading = !digitalRead(ResetKey);

      if(!debouncereading && debouncecounter > 0)
      {
        debouncecounter--;
      }
      if(debouncereading)
      {
        debouncecounter++; 
      }
      if(debouncecounter >= debounce_count)
      {
        btime = millis();
        debouncecounter = 0;
        ResetHeld = 1;
      }
    delay (10); // wait 10ms
    }
   } else {
    ResetHeld = 0;
   }
return ResetHeld;
}


void SaveSlot(int myLang,int myHz) {
  // Save Current parameter into EPROM
  EEPROM.write(1,myLang);
  EEPROM.write(2,myHz);
}

void FlashLED(int flashcount) {
    // Light the LED to represent which language is selected 0:EU 1:US 2:JP
    switch (flashcount) {
    case 0:
      digitalWrite(LEDr, LOW);
      digitalWrite(LEDg, HIGH);
      digitalWrite(LEDb, LOW);
      break;
    case 1:
      digitalWrite(LEDr, LOW);
      digitalWrite(LEDg, LOW);
      digitalWrite(LEDb, HIGH);
      break;
    case 2:
      digitalWrite(LEDr, HIGH);
      digitalWrite(LEDg, LOW);
      digitalWrite(LEDb, LOW);
      break;
  }
}

void LEDoff() {
      digitalWrite(LEDr, LOW);
      digitalWrite(LEDg, LOW);
      digitalWrite(LEDb, LOW);
}

void SetHZ(int myHz) {
    if (myHz==0){
      digitalWrite(ModHz, LOW);   //GND 50HZ
    } else {
      digitalWrite(ModHz, HIGH);  //5V 60HZ
    }
}

void SetRegion(int myLang) {
        if (CurrentLang==0){
          CurrentHz=0;
          digitalWrite(ModLang, HIGH);//5V ENG
        } else if (CurrentLang==1){
          digitalWrite(ModLang, HIGH);//5V ENG
          CurrentHz=1;
        } else {
          digitalWrite(ModLang, LOW);//GND JAP
          CurrentHz=1;
        }
}
